# Auto-generated by DjangoCFG - see CLAUDE.md

from __future__ import annotations

from typing import Any

from pydantic import BaseModel, ConfigDict, Field

from ..enums import ServiceControlRequestAction


class ServiceControlRequest(BaseModel):
    """
    Serializer for service control commands.

    Request model (no read-only fields).
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra="allow",
        frozen=False,
    )

    action: ServiceControlRequestAction = Field(description='Control action to perform  * `st...')
    params: dict[str, Any] | None = None
    force: bool | None = None



